## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import datetime
from zope.component import queryUtility
from zope.interface.verify import verifyObject, verifyClass
from hurry.workflow.interfaces import IWorkflowState
from waeup.kofa.interfaces import ICSVExporter
from waeup.kofa.schoolgrades import ResultEntry
from waeup.kofa.students.tests.test_batching import StudentImportExportSetup

from kofacustom.nigeria.utils.utils import NigeriaKofaUtils
from kofacustom.nigeria.students.export import (
    NigeriaStudentExporter, NigeriaStudentPaymentExporter,
    ClearanceRequestedStudentExporter, NigeriaDataForBursaryExporter,
    NigeriaTrimmedDataExporter)
from kofacustom.nigeria.testing import FunctionalLayer

curr_year = datetime.datetime.now().year

class NigeriaStudentExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(NigeriaStudentExporterTest, self).setUp()
        self.setup_for_export()
        result_entry = ResultEntry(
            sorted(NigeriaKofaUtils.EXAM_SUBJECTS_DICT.keys())[0],
            sorted(NigeriaKofaUtils.EXAM_GRADES)[0][0]
            )
        self.student.alr_results = [
            result_entry]
        self.student.fst_sit_results = [
            result_entry]
        self.student.scd_sit_results = [
            result_entry]
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = NigeriaStudentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, NigeriaStudentExporter)
        return


    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = NigeriaStudentExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertMatches(
            'adm_code,alr_date,alr_fname,alr_no,alr_results,'
            'clr_code,date_of_birth,def_adm,disabled,email,emp2_end,'
            'emp2_position,emp2_reason,emp2_start,emp_end,emp_position,'
            'emp_reason,emp_start,employer,employer2,'
            'financial_clearance_date,financially_cleared_by,'
            'firstname,flash_notice,former_matric,'
            'fst_sit_date,fst_sit_fname,fst_sit_no,fst_sit_results,'
            'fst_sit_type,hq2_degree,hq2_disc,hq2_matric_no,hq2_school,'
            'hq2_session,hq2_type,hq_degree,hq_disc,hq_fname,hq_matric_no,'
            'hq_school,hq_session,hq_type,is_staff,lastname,lga,'
            'marit_stat,matric_number,middlename,nationality,'
            'next_kin_address,next_kin_name,next_kin_phone,next_kin_relation,'
            'nysc_lga,nysc_location,nysc_year,officer_comment,parents_email,'
            'perm_address,personal_updated,phone,physical_clearance_date,'
            'provisionally_cleared,reg_number,'
            'religion,scd_sit_date,scd_sit_fname,scd_sit_no,'
            'scd_sit_results,scd_sit_type,sex,student_id,'
            'suspended,suspended_comment,password,state,history,certcode,is_postgrad,'
            'current_level,current_session\r\nmy adm code,,,,'
            '"[(\'accounts\', \'A\')]",my clr code,1981-02-04#,,,'
            'anna@sample.com,,,,,,,,,,,,,Anna,,,,,,"[(\'accounts\', \'A\')]"'
            ',,,,,,,,,,,,,,,,Tester,,,234,M.,NG,,,,,,,,,,'
            '"Studentroad 21\nLagos 123456\n",,+234-123-12345#,,,123,,,,,'
            '"[(\'accounts\', \'A\')]",,f,A111111,0,,,created,'
            '[u\'2012-11-06 13:16:41 WAT - Record created by system\'],'
            'CERT1,0,200,2012\r\n',
            result
            )
        return

class NigeriaStudentPaymentExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(NigeriaStudentPaymentExporterTest, self).setUp()
        self.setup_for_export()
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = NigeriaStudentPaymentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, NigeriaStudentPaymentExporter)
        return

    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        self.student['payments']['my-payment'].r_company = 'interswitch'
        self.student['payments']['my-payment'].r_card_num = '789'
        exporter = NigeriaStudentPaymentExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertMatches(
            'ac,amount_auth,creation_date,gateway_amt,net_amt,p_category,p_combi,p_current,'
            'p_id,p_item,p_level,p_option,p_session,p_split_data,p_state,payment_date,provider_amt,'
            'r_amount_approved,r_card_num,r_code,r_company,r_desc,'
            'r_pay_reference,r_payment_link,thirdparty_amt,student_id,state,'
            'current_session\r\n'
            '666,12.12,2012-04-01 13:12:01#,,,schoolfee,[],1,my-id,p-item,'
            '100,,%s,,paid,2012-04-01 14:12:01#,,12.12,'
            '789,r-code,interswitch,,,,,A111111,created,2012\r\n' % (curr_year-6),
            result
            )
        return

class ClearanceRequestedStudentExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(ClearanceRequestedStudentExporterTest, self).setUp()
        self.setup_for_export()
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = ClearanceRequestedStudentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, ClearanceRequestedStudentExporter)
        return

    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        self.student.physical_clearance_date = 'Come soon'
        IWorkflowState(self.student).setState('clearance requested')
        exporter = ClearanceRequestedStudentExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertMatches(
            'student_id,reg_number,display_fullname,state,current_session,'
            'history,physical_clearance_date,email,phone\r\n'
            'A111111,123,Anna M. Tester,clearance requested,'
            '2012,2014-12-02 07:17:02 WAT - Record created by system,'
            'Come soon,anna@sample.com,+234-123-12345\r\n',
            result
            )
        return

class BursaryDataExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(BursaryDataExporterTest, self).setUp()
        self.setup_for_export()
        return

    def test_export_all(self):
        # we can really export all payments
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = NigeriaDataForBursaryExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'ac,amount_auth,creation_date,formatted_p_date,gateway_amt,net_amt,'
            'p_category,p_combi,p_current,p_id,p_item,p_level,p_option,p_session,p_split_data,p_state,'
            'payment_date,provider_amt,r_amount_approved,r_card_num,r_code,'
            'r_company,r_desc,r_pay_reference,r_payment_link,thirdparty_amt,student_id,'
            'matric_number,reg_number,firstname,middlename,lastname,sex,state,'
            'current_session,entry_session,entry_mode,faccode,depcode,certcode,lga\r\n'

            '666,12.12,%s-04-01 13:12:01#,,,,schoolfee,[],1,my-id,p-item,'
            '100,,%s,,paid,%s-04-01 14:12:01#,,12.12,,r-code,,,,,,A111111,'
            '234,123,Anna,M.,Tester,f,created,2012,2010,ug_ft,NA,NA,CERT1,\r\n'
            % (curr_year-6, curr_year-6, curr_year-6)
            )
        return

class NigeriaStudentTrimmedDataExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    std_csv_entry = (
        'my adm code,my clr code,1981-02-04#,anna@sample.com,,'
        'Anna,,Tester,234,M.,NG,,,"Studentroad 21\nLagos 123456\n",,'
        '+234-123-12345#,123,f,A111111,0,,,created'
        )

    def setUp(self):
        super(NigeriaStudentTrimmedDataExporterTest, self).setUp()
        self.setup_for_export()
        self.student.lga = 'abia_aba_north'
        return

    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = NigeriaTrimmedDataExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertTrue(
            'student_id,matric_number,reg_number,firstname,middlename,'
            'lastname,sex,email,phone,nationality,date_of_birth,state,'
            'current_mode,certcode,faccode,depcode,current_level,'
            'current_session,current_verdict,entry_session,lg_state,lg_area\r\n'
            'A111111,234,123,Anna,M.,Tester,f,anna@sample.com,+234-123-12345#,'
            'NG,1981-02-04#,created,ug_ft,CERT1,NA,NA,200,2012,0,2010,'
            'abia,aba-north'
            in result
            )
        return
